<?php

namespace Drupal\commerce_eta\Entity;

use Drupal\commerce\ConditionGroup;
use Drupal\commerce\Plugin\Commerce\Condition\ParentEntityAwareInterface;
use Drupal\commerce_eta\OrderConditions;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Defines the Trigger entity.
 *
 * @ConfigEntityType(
 *   id = "commerce_event_trigger",
 *   label = @Translation("Trigger"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\commerce_eta\TriggerListBuilder",
 *     "form" = {
 *       "add" = "Drupal\commerce_eta\Form\TriggerForm",
 *       "edit" = "Drupal\commerce_eta\Form\TriggerForm",
 *       "delete" = "Drupal\commerce_eta\Form\TriggerDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\commerce_eta\TriggerHtmlRouteProvider",
 *     },
 *   },
 *   config_prefix = "commerce_event_trigger",
 *   admin_permission = "administer site configuration",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid",
 *     "cron_status" = "cron_status",
 *     "target_entity_type" = "target_entity_type",
 *     "conditions",
 *     "conditionOperator",
 *   },
 *   links = {
 *     "canonical" = "/admin/commerce/config/events/trigger/{commerce_event_trigger}",
 *     "add-form" = "/admin/commerce/config/events/trigger/add",
 *     "edit-form" = "/admin/commerce/config/events/trigger/{commerce_event_trigger}/edit",
 *     "delete-form" = "/admin/commerce/config/events/trigger/{commerce_event_trigger}/delete",
 *     "collection" = "/admin/commerce/config/events/triggers"
 *   }
 * )
 */
class Trigger extends ConfigEntityBase implements TriggerInterface {

  /**
   * The Trigger ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The Trigger label.
   *
   * @var string
   */
  protected $label;

  /**
   * @var boolean
   */
  protected $cron_status;

  /**
   * @var boolean
   */
  protected $log_status;

  /**
   * @var integer
   */
  protected $run_limit;

  /**
   * @var string
   */
  protected $target_entity_type;

  /**
   * The conditions.
   *
   * @var array
   */
  protected $conditions = [];

  /**
   * The condition operator.
   *
   * @var string
   */
  protected $conditionOperator = 'AND';

  /**
   * @return bool
   */
  public function getCronStatus() {
    return $this->cron_status;
  }



  /**
   * @param \Drupal\commerce_eta\Entity\boolean $status
   *
   * @return \Drupal\commerce_eta\Entity\boolean
   */
  public function setCronStatus(boolean $status) {
    return $this->cron_status = $status;
  }

  public function getLogStatus() {
    return $this->log_status;
  }

  public function setLogStatus(boolean $status) {
    return $this->log_status = $status;
  }

  public function getRunLimit() {
    return $this->run_limit;
  }

  public function setRunLimit(integer $limit) {
    return $this->run_limit = $limit;
  }

  /**
   * @return string
   */
  public function getTargetEntityType() {
    return $this->target_entity_type;
  }

  /**
   * @param \Drupal\Core\Config\Entity\ConfigEntityBase $entity_type
   *
   * @return int|string|null
   */
  public function setTargetEntityType(\Drupal\Core\Config\Entity\ConfigEntityBase $entity_type) {
    return $this->target_entity_type = $entity_type->id();
  }

  /**
   * {@inheritdoc}
   */
  public function getConditions() {
    $plugin_manager = \Drupal::service('plugin.manager.commerce_condition');
    $conditions = [];
    foreach ($this->conditions as $condition) {
      $condition = $plugin_manager->createInstance($condition['plugin'], $condition['configuration']);
      if ($condition instanceof ParentEntityAwareInterface) {
        $condition->setParentEntity($this);
      }
      $conditions[] = $condition;
    }
    return $conditions;
  }

  /**
   * {@inheritdoc}
   */
  public function getConditionOperator() {
    return $this->conditionOperator;
  }

  /**
   * {@inheritdoc}
   */
  public function setConditionOperator($condition_operator) {
    $this->conditionOperator = $condition_operator;
    return $this;
  }

  public function getStatus() {
    return $this->status;
  }

  public function setStatus($status) {
    return $this->status = $status;
  }

  public function isPublished() {
    return $this->status;
  }

  /**
   * {@inheritdoc}
   */
  public function applies(OrderInterface $order) {
    $conditions = $this->getConditions();
    if (!$conditions) {
      // Payment gateways without conditions always apply.
      return TRUE;
    }
    $order_conditions = array_filter($conditions, function ($condition) {
      /** @var \Drupal\commerce\Plugin\Commerce\Condition\ConditionInterface $condition */
      return $condition->getEntityTypeId() == 'commerce_order';
    });
    $order_conditions = new ConditionGroup($order_conditions, $this->getConditionOperator());

    return $order_conditions->evaluate($order);
  }

}
